@@ -333,6 +333,16 @@ impl FromCursor for RowsMetadata {
             None
         };
 
+        if flags.contains(RowsMetadataFlags::NO_METADATA) {
+            return Ok(RowsMetadata {
+                flags,
+                columns_count,
+                paging_state,
+                global_table_spec: None,
+                col_specs: vec![],
+            });
+        }
+
         let has_global_table_space = flags.contains(RowsMetadataFlags::GLOBAL_TABLE_SPACE);
         let global_table_spec = extract_global_table_space(cursor, has_global_table_space)?;
 
@@ -517,7 +527,7 @@ impl TryFrom<i16> for ColType {
             0x0030 => Ok(ColType::Udt),
             0x0031 => Ok(ColType::Tuple),
             0x0080 => Ok(ColType::Varchar),
-            _ => Err("Unexpected column type".into()),
+            _ => Err(format!("Unexpected column type {:?}", value).into()),
         }
     }
 }
@@ -790,9 +800,6 @@ impl Serialize for PreparedMetadataFlags {
 /// The structure that represents metadata of prepared response.
 #[derive(Debug, Clone, PartialEq, Ord, PartialOrd, Eq, Hash)]
 pub struct PreparedMetadata {
-    pub flags: PreparedMetadataFlags,
-    pub columns_count: i32,
-    pub pk_count: i32,
     pub pk_indexes: Vec<i16>,
     pub global_table_spec: Option<TableSpec>,
     pub col_specs: Vec<ColSpec>,
@@ -801,23 +808,19 @@ pub struct PreparedMetadata {
 impl Serialize for PreparedMetadata {
     #[inline]
     fn serialize(&self, cursor: &mut Cursor<&mut Vec<u8>>) {
-        match self
-            .flags
-            .contains(PreparedMetadataFlags::GLOBAL_TABLE_SPACE)
-        {
-            false => {
-                assert!(self.global_table_spec.is_none());
-                assert!(!self.col_specs.is_empty());
-            }
-            true => {
-                assert!(self.global_table_spec.is_some());
-                assert!(self.col_specs.is_empty());
-            }
+        if self.global_table_spec.is_some() {
+            PreparedMetadataFlags::GLOBAL_TABLE_SPACE
+        } else {
+            PreparedMetadataFlags::empty()
         }
+        .serialize(cursor);
+
+        let columns_count = self.col_specs.len() as i32;
+        columns_count.serialize(cursor);
+
+        let pk_count = self.pk_indexes.len() as i32;
+        pk_count.serialize(cursor);
 
-        self.flags.serialize(cursor);
-        self.columns_count.serialize(cursor);
-        self.pk_count.serialize(cursor);
         self.pk_indexes.iter().for_each(|f| f.serialize(cursor));
 
         if let Some(global_table_spec) = &self.global_table_spec {
@@ -857,9 +860,6 @@ impl PreparedMetadata {
         let col_specs = ColSpec::parse_colspecs(cursor, columns_count, has_global_table_space)?;
 
         Ok(PreparedMetadata {
-            flags,
-            columns_count,
-            pk_count,
             pk_indexes,
             global_table_spec,
             col_specs,
@@ -1178,6 +1178,7 @@ mod rows_metadata {
 
         let expected = RowsMetadata {
             flags: RowsMetadataFlags::empty(),
+
             columns_count: 2,
             paging_state: None,
             global_table_spec: None,
@@ -1290,14 +1291,14 @@ mod rows {
         let bytes = &[
             0, 0, 0, 2, // rows flag
             0, 0, 0, 4, // rows metadataflag
-            0, 0, 0, 0, // columns count
+            0, 0, 0, 3, // columns count
             0, 0, 0, 0, // rows count
         ];
 
         let expected = ResResultBody::Rows(BodyResResultRows {
             metadata: RowsMetadata {
                 flags: RowsMetadataFlags::NO_METADATA,
-                columns_count: 0,
+                columns_count: 3,
                 paging_state: None,
                 global_table_spec: None,
                 col_specs: vec![],
@@ -1355,9 +1356,6 @@ mod prepared_metadata {
         ];
 
         let expected = PreparedMetadata {
-            flags: PreparedMetadataFlags::empty(),
-            columns_count: 2,
-            pk_count: 1,
             pk_indexes: vec![0],
             global_table_spec: None,
             col_specs: vec![
@@ -1446,9 +1444,6 @@ mod prepared {
         let expected = ResResultBody::Prepared(BodyResResultPrepared {
             id: CBytesShort::new(to_short(1)),
             metadata: PreparedMetadata {
-                flags: PreparedMetadataFlags::empty(),
-                columns_count: 2,
-                pk_count: 1,
                 pk_indexes: vec![0],
                 global_table_spec: None,
                 col_specs: vec![
