@@ -55,6 +55,126 @@ macro_rules! list_as_rust {
     );
 }
 
+macro_rules! list_as_cassandra_type {
+    () => {
+        impl crate::types::AsCassandraType for List {
+            fn as_cassandra_type(
+                &self,
+            ) -> Result<Option<crate::types::cassandra_type::CassandraType>> {
+                use crate::error::Error;
+                use crate::types::cassandra_type::wrapper_fn;
+                use crate::types::cassandra_type::CassandraType;
+                use std::ops::Deref;
+
+                match self.metadata.value {
+                    Some(ColTypeOptionValue::CList(ref type_option))
+                    | Some(ColTypeOptionValue::CSet(ref type_option)) => {
+                        let type_option_ref = type_option.deref().clone();
+                        let wrapper = wrapper_fn(&type_option_ref.id);
+                        let convert = self.map(|bytes| wrapper(bytes, &type_option_ref).unwrap());
+                        Ok(Some(CassandraType::List(convert)))
+                    }
+                    _ => Err(Error::General(format!(
+                        "Invalid conversion. \
+                            Cannot convert {:?} into List (valid types: List, Set).",
+                        self.metadata.value
+                    ))),
+                }
+            }
+        }
+    };
+}
+
+macro_rules! map_as_cassandra_type {
+    () => {
+        impl crate::types::AsCassandraType for Map {
+            fn as_cassandra_type(
+                &self,
+            ) -> Result<Option<crate::types::cassandra_type::CassandraType>> {
+                use crate::types::cassandra_type::wrapper_fn;
+                use crate::types::cassandra_type::CassandraType;
+                use std::ops::Deref;
+
+                if let Some(ColTypeOptionValue::CMap(
+                    ref key_col_type_option,
+                    ref value_col_type_option,
+                )) = self.metadata.value
+                {
+                    let key_col_type_option = key_col_type_option.deref().clone();
+                    let value_col_type_option = value_col_type_option.deref().clone();
+
+                    let key_wrapper = wrapper_fn(&key_col_type_option.id);
+
+                    let value_wrapper = wrapper_fn(&value_col_type_option.id);
+
+                    let map = self
+                        .data
+                        .iter()
+                        .map(|(key, value)| {
+                            (
+                                key_wrapper(key, &key_col_type_option).unwrap(),
+                                value_wrapper(value, &value_col_type_option).unwrap(),
+                            )
+                        })
+                        .collect::<Vec<(CassandraType, CassandraType)>>();
+
+                    return Ok(Some(CassandraType::Map(map)));
+                } else {
+                    panic!("not  amap")
+                }
+            }
+        }
+    };
+}
+
+macro_rules! tuple_as_cassandra_type {
+    () => {
+        impl crate::types::AsCassandraType for Tuple {
+            fn as_cassandra_type(
+                &self,
+            ) -> Result<Option<crate::types::cassandra_type::CassandraType>> {
+                use crate::types::cassandra_type::wrapper_fn;
+                use crate::types::cassandra_type::CassandraType;
+
+                let values = self
+                    .data
+                    .iter()
+                    .map(|(col_type, bytes)| {
+                        let wrapper = wrapper_fn(&col_type.id);
+                        wrapper(&bytes, col_type).unwrap()
+                    })
+                    .collect();
+
+                Ok(Some(CassandraType::Tuple(values)))
+            }
+        }
+    };
+}
+
+macro_rules! udt_as_cassandra_type {
+    () => {
+        impl crate::types::AsCassandraType for Udt {
+            fn as_cassandra_type(
+                &self,
+            ) -> Result<Option<crate::types::cassandra_type::CassandraType>> {
+                use crate::types::cassandra_type::wrapper_fn;
+                use crate::types::cassandra_type::CassandraType;
+                use std::collections::HashMap;
+
+                let mut map = HashMap::new();
+
+                self.data.iter().for_each(|(key, (col_type, bytes))| {
+                    let wrapper = wrapper_fn(&col_type.id);
+                    let value = wrapper(&bytes, col_type).unwrap();
+                    map.insert(key.clone(), value);
+                });
+
+                Ok(Some(CassandraType::Udt(map)))
+            }
+        }
+    };
+}
+
 macro_rules! map_as_rust {
     ({ $($key_type:tt)+ }, { $($val_type:tt)+ }) => (
         impl AsRustType<HashMap<$($key_type)+, $($val_type)+>> for Map {
@@ -153,17 +273,18 @@ macro_rules! as_res_opt {
 /// plus the matching Rust type.
 macro_rules! as_rust_type {
     ($data_type_option:ident, $data_value:ident, Blob) => {
+
         match $data_type_option.id {
             ColType::Blob => as_res_opt!($data_value, decode_blob),
             ColType::Custom => {
                 let unmarshal = || {
-                    if let Some(ColTypeOptionValue::CString(value)) = &$data_type_option.value {
+                    if let Some(crate::frame::frame_result::ColTypeOptionValue::CString(value)) = &$data_type_option.value {
                         if value.as_str() == "org.apache.cassandra.db.marshal.BytesType" {
                             return as_res_opt!($data_value, decode_blob);
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into Vec<u8> (valid types: org.apache.cassandra.db.marshal.BytesType).",
                         $data_type_option
@@ -172,7 +293,7 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
                  Cannot convert {:?} into Vec<u8> (valid types: Blob).",
                 $data_type_option.id
@@ -188,7 +309,7 @@ macro_rules! as_rust_type {
             // it's not mentioned in
             // https://github.com/apache/cassandra/blob/trunk/doc/native_protocol_v4.spec#L582
             // ColType::XXX => decode_text($data_value)?
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
                  Cannot convert {:?} into String (valid types: Custom, Ascii, Varchar).",
                 $data_type_option.id
@@ -206,7 +327,7 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into bool (valid types: org.apache.cassandra.db.marshal.BooleanType).",
                         $data_type_option
@@ -215,9 +336,9 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into bool (valid types: Boolean).",
+                 Cannot convert {:?} into bool (valid types: Boolean, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -239,7 +360,7 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into i64 (valid types: org.apache.cassandra.db.marshal.{{LongType|IntegerType|CounterColumnType|TimestampType|TimeType}}).",
                         $data_type_option
@@ -248,10 +369,10 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into i64 (valid types: Bigint, Timestamp, Time, Variant,\
-                 Counter).",
+                 Cannot convert {:?} into i64 (valid types: Bigint, Timestamp, Time,\
+                 Counter, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -262,7 +383,7 @@ macro_rules! as_rust_type {
             ColType::Date => as_res_opt!($data_value, decode_date),
             ColType::Custom => {
                 let unmarshal = || {
-                    if let Some(ColTypeOptionValue::CString(value)) = &$data_type_option.value {
+                    if let Some(crate::frame::frame_result::ColTypeOptionValue::CString(value)) = &$data_type_option.value {
                         match value.as_str() {
                             "org.apache.cassandra.db.marshal.Int32Type" => return as_res_opt!($data_value, decode_int),
                             "org.apache.cassandra.db.marshal.SimpleDateType" => return as_res_opt!($data_value, decode_date),
@@ -270,7 +391,7 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into i32 (valid types: org.apache.cassandra.db.marshal.Int32Type).",
                         $data_type_option
@@ -279,9 +400,9 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into i32 (valid types: Int, Date).",
+                 Cannot convert {:?} into i32 (valid types: Int, Date, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -297,7 +418,7 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into i16 (valid types: org.apache.cassandra.db.marshal.ShortType).",
                         $data_type_option
@@ -306,9 +427,9 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into i16 (valid types: Smallint).",
+                 Cannot convert {:?} into i16 (valid types: Smallint, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -324,7 +445,7 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into i8 (valid types: org.apache.cassandra.db.marshal.ByteType).",
                         $data_type_option
@@ -333,9 +454,9 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into i8 (valid types: Tinyint).",
+                 Cannot convert {:?} into i8 (valid types: Tinyint, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -375,8 +496,8 @@ macro_rules! as_rust_type {
             }
             _ => Err(Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into i64 (valid types: Bigint, Timestamp, Time, Variant,\
-                 Counter).",
+                 Cannot convert {:?} into NonZeroI64 (valid types: Bigint, Timestamp, Time,\
+                 Counter, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -410,7 +531,7 @@ macro_rules! as_rust_type {
             }
             _ => Err(Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into i32 (valid types: Int, Date).",
+                 Cannot convert {:?} into NonZeroI32 (valid types: Int, Date, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -429,7 +550,7 @@ macro_rules! as_rust_type {
 
                     Err(Error::General(format!(
                         "Invalid conversion. \
-                         Cannot convert marshaled type {:?} into i16 (valid types: org.apache.cassandra.db.marshal.ShortType).",
+                         Cannot convert marshaled type {:?} into NonZeroI16 (valid types: org.apache.cassandra.db.marshal.ShortType).",
                         $data_type_option
                     )))
                 };
@@ -438,7 +559,7 @@ macro_rules! as_rust_type {
             }
             _ => Err(Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into i16 (valid types: Smallint).",
+                 Cannot convert {:?} into NonZeroI16 (valid types: Smallint, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -458,7 +579,7 @@ macro_rules! as_rust_type {
 
                     Err(Error::General(format!(
                         "Invalid conversion. \
-                         Cannot convert marshaled type {:?} into i8 (valid types: org.apache.cassandra.db.marshal.ByteType).",
+                         Cannot convert marshaled type {:?} into NonZeroI8 (valid types: org.apache.cassandra.db.marshal.ByteType).",
                         $data_type_option
                     )))
                 };
@@ -467,7 +588,7 @@ macro_rules! as_rust_type {
             }
             _ => Err(Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into i8 (valid types: Tinyint).",
+                 Cannot convert {:?} into NonZeroI8 (valid types: Tinyint, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -483,7 +604,7 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into f64 (valid types: org.apache.cassandra.db.marshal.DoubleType).",
                         $data_type_option
@@ -492,9 +613,9 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into f64 (valid types: Double).",
+                 Cannot convert {:?} into f64 (valid types: Double, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -510,7 +631,7 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into f32 (valid types: org.apache.cassandra.db.marshal.FloatType).",
                         $data_type_option
@@ -519,9 +640,9 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into f32 (valid types: Float).",
+                 Cannot convert {:?} into f32 (valid types: Float, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -537,7 +658,7 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into IpAddr (valid types: org.apache.cassandra.db.marshal.InetAddressType).",
                         $data_type_option
@@ -546,9 +667,9 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into IpAddr (valid types: Inet).",
+                 Cannot convert {:?} into IpAddr (valid types: Inet, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -565,7 +686,7 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
                          Cannot convert marshaled type {:?} into Uuid (valid types: org.apache.cassandra.db.marshal.{{UUIDType|TimeUUIDType}}).",
                         $data_type_option
@@ -574,9 +695,9 @@ macro_rules! as_rust_type {
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into Uuid (valid types: Uuid, Timeuuid).",
+                 Cannot convert {:?} into Uuid (valid types: Uuid, Timeuuid, Custom).",
                 $data_type_option.id
             ))),
         }
@@ -589,7 +710,7 @@ macro_rules! as_rust_type {
                     .map_err(Into::into),
                 None => Ok(None),
             },
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
                  Cannot convert {:?} into List (valid types: List, Set).",
                 $data_type_option.id
@@ -642,7 +763,7 @@ macro_rules! as_rust_type {
             },
             _ => Err(Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into Tuple (valid types: tuple).",
+                 Cannot convert {:?} into Tuple (valid types: Tuple).",
                 $data_type_option.id
             ))),
         }
@@ -673,7 +794,7 @@ macro_rules! as_rust_type {
                 Some(ref bytes) => decode_decimal(bytes).map(Some).map_err(Into::into),
                 None => Ok(None),
             },
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
                  Cannot convert {:?} into Decimal (valid types: Decimal).",
                 $data_type_option.id
@@ -722,6 +843,9 @@ macro_rules! as_rust_type {
     };
     ($data_type_option:ident, $data_value:ident, BigInt) => {
         match $data_type_option.id {
+            ColType::Varint => {
+                as_res_opt!($data_value, decode_varint)
+            }
             ColType::Custom => {
                 let unmarshal = || {
                     if let Some(ColTypeOptionValue::CString(value)) = &$data_type_option.value {
@@ -730,19 +854,18 @@ macro_rules! as_rust_type {
                         }
                     }
 
-                    Err(Error::General(format!(
+                    Err(crate::error::Error::General(format!(
                         "Invalid conversion. \
-                         Cannot convert marshaled type {:?} into BigInt (valid types: org.apache.cassandra.db.marshal.IntegerType).",
+                         Cannot convert marshalled type {:?} into BigInt (valid types: org.apache.cassandra.db.marshal.IntegerType).",
                         $data_type_option
                     )))
                 };
 
                 unmarshal()
             }
-            _ => Err(Error::General(format!(
+            _ => Err(crate::error::Error::General(format!(
                 "Invalid conversion. \
-                 Cannot convert {:?} into i64 (valid types: Bigint, Timestamp, Time, Variant,\
-                 Counter).",
+                 Cannot convert {:?} into BigInt (valid types: Varint, Custom)",
                 $data_type_option.id
             ))),
         }
